@@ -8,15 +8,12 @@ pub mod server;
 
 pub mod acceptor;
 pub mod acme;
-pub mod adapter;
-pub mod repo;
+pub mod router;
 pub mod stream;
 
 pub use acceptor::*;
 pub use acme::*;
-pub use adapter::ServerMeshAdapter;
 pub use builder::ServerBuilder;
 pub use conf::ServerConf;
-pub use repo::*;
 pub use server::Server;
 pub use stream::*;
