@@ -1,7 +1,9 @@
 mod all;
 mod core;
 mod web;
+mod auth;
 
 pub use self::core::*;
 pub use all::*;
 pub use web::*;
+pub use auth::*;
\ No newline at end of file
